use strict;
use warnings;
use Test::More;

use_ok 'Data::Object::Role::Array';

can_ok 'Data::Object::Role::Array', 'all';
can_ok 'Data::Object::Role::Array', 'any';
can_ok 'Data::Object::Role::Array', 'clear';
can_ok 'Data::Object::Role::Array', 'count';
can_ok 'Data::Object::Role::Array', 'data';
can_ok 'Data::Object::Role::Array', 'defined';
can_ok 'Data::Object::Role::Array', 'delete';
can_ok 'Data::Object::Role::Array', 'detract';
can_ok 'Data::Object::Role::Array', 'dump';
can_ok 'Data::Object::Role::Array', 'each';
can_ok 'Data::Object::Role::Array', 'each_key';
can_ok 'Data::Object::Role::Array', 'each_n_values';
can_ok 'Data::Object::Role::Array', 'each_value';
can_ok 'Data::Object::Role::Array', 'empty';
can_ok 'Data::Object::Role::Array', 'eq';
can_ok 'Data::Object::Role::Array', 'exists';
can_ok 'Data::Object::Role::Array', 'first';
can_ok 'Data::Object::Role::Array', 'ge';
can_ok 'Data::Object::Role::Array', 'get';
can_ok 'Data::Object::Role::Array', 'grep';
can_ok 'Data::Object::Role::Array', 'gt';
can_ok 'Data::Object::Role::Array', 'hash';
can_ok 'Data::Object::Role::Array', 'hashify';
can_ok 'Data::Object::Role::Array', 'head';
can_ok 'Data::Object::Role::Array', 'invert';
can_ok 'Data::Object::Role::Array', 'iterator';
can_ok 'Data::Object::Role::Array', 'join';
can_ok 'Data::Object::Role::Array', 'keyed';
can_ok 'Data::Object::Role::Array', 'keys';
can_ok 'Data::Object::Role::Array', 'last';
can_ok 'Data::Object::Role::Array', 'le';
can_ok 'Data::Object::Role::Array', 'length';
can_ok 'Data::Object::Role::Array', 'list';
can_ok 'Data::Object::Role::Array', 'lt';
can_ok 'Data::Object::Role::Array', 'map';
can_ok 'Data::Object::Role::Array', 'max';
can_ok 'Data::Object::Role::Array', 'methods';
can_ok 'Data::Object::Role::Array', 'min';
can_ok 'Data::Object::Role::Array', 'ne';
can_ok 'Data::Object::Role::Array', 'none';
can_ok 'Data::Object::Role::Array', 'nsort';
can_ok 'Data::Object::Role::Array', 'one';
can_ok 'Data::Object::Role::Array', 'pairs';
can_ok 'Data::Object::Role::Array', 'pairs_array';
can_ok 'Data::Object::Role::Array', 'pairs_hash';
can_ok 'Data::Object::Role::Array', 'part';
can_ok 'Data::Object::Role::Array', 'pop';
can_ok 'Data::Object::Role::Array', 'print';
can_ok 'Data::Object::Role::Array', 'push';
can_ok 'Data::Object::Role::Array', 'random';
can_ok 'Data::Object::Role::Array', 'reverse';
can_ok 'Data::Object::Role::Array', 'rnsort';
can_ok 'Data::Object::Role::Array', 'roles';
can_ok 'Data::Object::Role::Array', 'rotate';
can_ok 'Data::Object::Role::Array', 'rsort';
can_ok 'Data::Object::Role::Array', 'say';
can_ok 'Data::Object::Role::Array', 'set';
can_ok 'Data::Object::Role::Array', 'shift';
can_ok 'Data::Object::Role::Array', 'size';
can_ok 'Data::Object::Role::Array', 'slice';
can_ok 'Data::Object::Role::Array', 'sort';
can_ok 'Data::Object::Role::Array', 'sum';
can_ok 'Data::Object::Role::Array', 'tail';
can_ok 'Data::Object::Role::Array', 'throw';
can_ok 'Data::Object::Role::Array', 'type';
can_ok 'Data::Object::Role::Array', 'unique';
can_ok 'Data::Object::Role::Array', 'unshift';
can_ok 'Data::Object::Role::Array', 'values';

ok 1 and done_testing;
